import React, { useEffect, useMemo, useRef, useState } from 'react';
import {
  select,
  selectAll,
  scaleLinear,
  CurveFactory,
  axisBottom,
  axisLeft,
  area,
  line,
  curveCardinal,
  curveStepAfter,
  curveNatural,
  curveLinear,
  min,
  max,
} from 'd3';
import { visualizationColors } from 'config';
import AxisComponent from '../Axis';

// Define the LineChartProps type
export type LineChartProps = {
  data: { x: number; y: number }[][]; // Array of arrays of data points
  xAxisLabel: string;
  yAxisLabel: string;
  showMarks: boolean;
  curveInterpolation: string[];
  isArea: boolean;
};

// LineChart component
const LineChart: React.FC<LineChartProps> = ({ data, xAxisLabel, yAxisLabel, showMarks, curveInterpolation, isArea }) => {
  // Ref for SVG element
  const svgRef = useRef<SVGSVGElement | null>(null);
  // State for dimensions of the SVG container
  const [dimensions, setDimensions] = useState({ width: 0, height: 0 });

  // Define margin
  const marginPercentage = {
    top: 0.245,
    right: 0.245,
    bottom: 0.245,
    left: 0.245,
  };

  const [state, setState] = useState({
    margin: {
      top: 0,
      right: 0,
      bottom: 0,
      left: 0,
    },
    innerWidth: 0,
    innerHeight: 0,
    xScale: scaleLinear(),
    yScale: scaleLinear(),
    color: visualizationColors.GPCat.colors[14],
    d3line: line<{ x: number; y: number }>(),
    d3area: area<{ x: number; y: number }>(),
  });

  // Effect to update dimensions when the component mounts or data changes
  useEffect(() => {
    if (!svgRef.current) return;

    // Get width and height of the SVG container
    const widthSVG: number = +svgRef.current.clientWidth;
    const heightSVG: number = +svgRef.current.clientHeight;

    // Update dimensions state
    setDimensions({ width: widthSVG, height: heightSVG });

    const margin = {
      top: marginPercentage.top * heightSVG,
      right: marginPercentage.right * heightSVG,
      bottom: marginPercentage.bottom * heightSVG,
      left: marginPercentage.left * heightSVG,
    };

    // Define innerwidth and innerheight
    const innerWidth = widthSVG - margin.left - margin.right;
    const innerHeight = heightSVG - margin.top - margin.bottom;

    // Remove existing elements from SVG
    // select(svgRef.current).selectAll('*').remove();

    // Create SVG container
    const svg = select(svgRef.current); //.attr('width', widthSVG).attr('height', heightSVG);

    // Create group for plotting area
    // const plotArea = svg.append('g').attr('transform', `translate(${margin.left},${margin.top})`);

    // IGNORE BELOW
    // Define scales
    const xScale = scaleLinear()
      .domain([min(data.flatMap((d) => d.map((p) => p.x))) || 0, max(data.flatMap((d) => d.map((p) => p.x))) || 0])
      .range([0, innerWidth]);

    const yScale = scaleLinear()
      .domain([min(data.flatMap((d) => d.map((p) => p.y))) || 0, max(data.flatMap((d) => d.map((p) => p.y))) || 0])
      .range([innerHeight, 0]);

    // Add grid lines for x axis
    svg.select<SVGGElement>('#lineplot-grid-lines-x').call(
      axisBottom(xScale)
        .tickSize(-innerHeight)
        .tickFormat(() => '')
        .tickSizeOuter(0)
        .ticks(xScale.ticks().length * 2),
    );

    // Add grid lines for y axis
    svg.select<SVGGElement>('#lineplot-grid-lines-y').call(
      axisLeft(yScale)
        .tickSize(-innerWidth)
        .tickFormat(() => '')
        .tickSizeOuter(0)
        .ticks(yScale.ticks().length / 2),
    );

    // CSS styling for grid lines
    selectAll('.grid line').style('stroke', visualizationColors.GPNeutral.colors[14][3]);

    // // Calculate suitable tick values
    // const xTicks = xScale.ticks();
    // const yTicks = yScale.ticks();

    // // Draw axes

    // const AxisTextCol = visualizationColors.GPNeutral.colors[14];
    // plotArea
    //   .append('g')
    //   .attr('class', 'x-axis')
    //   .attr('transform', `translate(0, ${innerHeight})`)
    //   .call(xAxis)
    //   .selectAll('.tick text')
    //   .attr('class', 'font-mono')
    //   .attr('fill', AxisTextCol[13])
    //   .style('stroke', 'none');

    // plotArea
    //   .append('g')
    //   .attr('class', 'y-axis')
    //   .call(yAxis)
    //   .selectAll('.tick text')
    //   .attr('class', 'font-mono')
    //   .attr('fill', AxisTextCol[13])
    //   .style('stroke', 'none');

    // Specify colors to use
    let d3area = area<{ x: number; y: number }>()
      .x((d) => xScale(d.x))
      .y0(innerHeight)
      .y1((d) => yScale(d.y));

    let d3line = line<{ x: number; y: number }>()
      .x((d) => xScale(d.x))
      .y((d) => yScale(d.y));

    const interpolationMap: Record<string, CurveFactory> = {
      smooth: curveCardinal,
      step: curveStepAfter,
      natural: curveNatural,
    };

    if (Array.isArray(curveInterpolation)) {
      curveInterpolation.forEach((option) => {
        const curveFunction = interpolationMap[option];
        if (curveFunction) {
          d3line = d3line.curve(curveFunction);
          d3area = d3area.curve(curveFunction);
        } else {
          d3line = d3line.curve(curveLinear);
          d3area = d3area.curve(curveLinear);
        }
      });
    } else {
      const curveFunction = interpolationMap[curveInterpolation];
      if (curveFunction) {
        d3line = d3line.curve(curveFunction);
        d3area = d3area.curve(curveFunction);
      } else {
        d3line = d3line.curve(curveLinear);
        d3area = d3area.curve(curveLinear);
      }
    }

    setState({
      margin,
      innerWidth,
      innerHeight,
      xScale,
      yScale,
      color: visualizationColors.GPCat.colors[14],
      d3line,
      d3area,
    });
  }, [data, dimensions.height, dimensions.width, showMarks, curveInterpolation, isArea]);

  // Return the LineChart component
  return (
    <svg
      ref={svgRef}
      className="container"
      width="100%"
      height="100%"
      preserveAspectRatio="xMidYMid meet"
      viewBox={`0 0 ${dimensions.width} ${dimensions.height}`}
    >
      <g transform={`translate(${state.margin.left}, ${state.margin.top})`}>
        <g id="lineplot-grid-lines-x" className="grid" transform={`translate(0, ${state.innerHeight})`}></g>
        <g id="lineplot-grid-lines-y" className="grid" transform={`translate(0, ${0})`}></g>
        <g id="xaxis" className='axis' transform={`translate(0, ${state.innerHeight})`}>
        <AxisComponent
          orientation="bottom"
          type={data.length > 5 ? 'categorical' : 'numerical'}
          range={[0, state.innerWidth]}
          domain={[min(data.flatMap((d) => d.map((p) => p.x))) || 0, max(data.flatMap((d) => d.map((p) => p.x))) || 0]}
        /></g>
        <g id="xaxis" className='axis' transform={`translate(${state.innerWidth}, 0)`}>
        <AxisComponent
          orientation="left"
          type={data.length > 1 ? 'categorical' : 'numerical'}
          range={[state.innerHeight, 0]}
          domain={[min(data.flatMap((d) => d.map((p) => p.y))) || 0, max(data.flatMap((d) => d.map((p) => p.y))) || 0]}
        />
        </g>
        <text className="x-axis-label font-Inter font-bold text-xs" x={state.innerWidth / 2} y={state.innerHeight + 38} textAnchor="middle">
          {xAxisLabel}
        </text>
        <text
          className="y-axis-label font-Inter font-bold text-xs"
          transform={`rotate(-90)`}
          x={-state.innerHeight / 2}
          y={-38}
          textAnchor="middle"
        >
          {yAxisLabel}
        </text>
        {isArea &&
          data.map((lineData, index) => (
            <path
              key={`area-${index}`}
              fill={state.color[index % state.color.length]}
              style={{ opacity: 0.3 }}
              d={state.d3area(lineData) || ''}
            />
          ))}
        {data.map((lineData, index) => {
          const colorIndex = index % state.color.length;
          return (
            <path key={`line-${index}`} fill="none" stroke={state.color[colorIndex]} strokeWidth={1.5} d={state.d3line(lineData) || ''} />
          );
        })}
        {showMarks &&
          data.map((lineData, index) => {
            const colorIndex = index % state.color.length;
            return lineData.map((d) => (
              <circle
                key={`${d.x}-${d.y} line mark-${index}`}
                className={`mark mark-${index}`}
                cx={state.xScale(d.x)}
                cy={state.yScale(d.y)}
                r={4.5}
                fill="white"
                stroke={state.color[colorIndex]}
                strokeWidth={1.5}
              />
            ));
          })}
      </g>
    </svg>
  );
};

export default LineChart;
